'use strict';

var shelljs = require('shelljs');
var fs = require('fs');
var path = require('path');
var childProcess = require('child_process');
var globby = require('globby');
var fixDir = __dirname + '/../fixture';
var tmpDir = __dirname + '/../fixture/tmp';
var removeVariance = require('./removeVariance');

shelljs.rm('-fR', tmpDir);
shelljs.mkdir('-p', tmpDir);
shelljs.touch(tmpDir + '/.empty');

describe('Invig', function () {
  describe('cli', function () {
    test('invigorates an entire directory', function () {
      var files = globby.sync([fixDir + '/*.coffee', fixDir + '/*.js']);
      files.forEach(function (src) {
        var base = path.basename(src);
        var dst = tmpDir + '/' + base;

        shelljs.cp('-f', path.dirname(src) + '/package.json', path.dirname(dst) + '/package.json');
        shelljs.cp('-f', src, dst);
        var cmd = 'env SCROLEX_MODE=passthru node ' + __dirname + '/cli.js --src "' + dst + '"';
        // console.log(cmd)
        var p = shelljs.exec(cmd);
        expect(removeVariance(p.stderr.trim())).toMatchSnapshot();
        expect(removeVariance(p.stdout.trim())).toMatchSnapshot();
        expect(p.code).toMatchSnapshot();
        expect(p.code).toBe(0);

        var result = fs.readFileSync(dst.replace(/\.coffee$/, '.js'), 'utf-8');
        expect(removeVariance(result.trim())).toMatchSnapshot();

        var pkg = fs.readFileSync(path.dirname(dst) + '/package.json', 'utf-8').trim();
        expect(pkg).toMatchSnapshot();
      });
    });
    test('invigorates via stdin', function (done) {
      var child = childProcess.spawn(process.argv[0], [__dirname + '/cli.js', '-qs', '-'], {
        env: {
          SCROLEX_MODE: 'passthru'
        }
      });

      child.stdin.write('console.log("hello")');
      child.stdin.end();

      var buf = '';
      child.stdout.on('data', function (data) {
        buf += '' + data;
      });

      child.on('close', function (code) {
        expect(removeVariance(buf)).toMatchSnapshot();
        done();
      });
    });
  });
});
//# sourceMappingURL=cli.test.js.map